@@ -461,6 +461,33 @@ def normalize_roles(roles):
     return meta.obj_list_to_dict(ret)
 
 
+def normalize_stacks(stacks):
+    """ Normalize Stack Object """
+    for stack in stacks:
+        stack['name'] = stack['stack_name']
+    return stacks
+
+
+def normalize_flavors(flavors):
+    """ Normalize a list of flavor objects """
+    for flavor in flavors:
+        flavor.pop('links', None)
+        flavor.pop('NAME_ATTR', None)
+        flavor.pop('HUMAN_ID', None)
+        flavor.pop('human_id', None)
+        if 'extra_specs' not in flavor:
+            flavor['extra_specs'] = {}
+        ephemeral = flavor.pop('OS-FLV-EXT-DATA:ephemeral', 0)
+        is_public = flavor.pop('os-flavor-access:is_public', True)
+        # Make sure both the extension version and a sane version are present
+        flavor['OS-FLV-EXT-DATA:ephemeral'] = ephemeral
+        flavor['ephemeral'] = ephemeral
+        flavor['os-flavor-access:is_public'] = is_public
+        flavor['is_public'] = is_public
+
+    return flavors
+
+
 def valid_kwargs(*valid_args):
     # This decorator checks if argument passed as **kwargs to a function are
     # present in valid_args.
