@@ -14,7 +14,6 @@ import jsonpatch
 
 from ironicclient import client as ironic_client
 from ironicclient import exceptions as ironic_exceptions
-from novaclient import exceptions as nova_exceptions
 
 from shade.exc import *  # noqa
 from shade import openstackcloud
@@ -747,13 +746,14 @@ class OperatorCloud(openstackcloud.OpenStackCloud):
                 _tasks.MachineNodeUpdate(node_id=uuid, patch=patch))
 
     @_utils.valid_kwargs('type', 'service_type', 'description')
-    def create_service(self, name, **kwargs):
+    def create_service(self, name, enabled=True, **kwargs):
         """Create a service.
 
         :param name: Service name.
         :param type: Service type. (type or service_type required.)
         :param service_type: Service type. (type or service_type required.)
         :param description: Service description (optional).
+        :param enabled: Whether the service is enabled (v3 only)
 
         :returns: a dict containing the services description, i.e. the
             following attributes::
@@ -767,17 +767,47 @@ class OperatorCloud(openstackcloud.OpenStackCloud):
             openstack API call.
 
         """
-        service_type = kwargs.get('type', kwargs.get('service_type'))
-        description = kwargs.get('description', None)
-        with _utils.shade_exceptions("Failed to create service {name}".format(
-                name=name)):
-            if self.cloud_config.get_api_version('identity').startswith('2'):
-                service_kwargs = {'service_type': service_type}
-            else:
-                service_kwargs = {'type': service_type}
+        type_ = kwargs.pop('type', None)
+        service_type = kwargs.pop('service_type', None)
 
-            service = self.manager.submitTask(_tasks.ServiceCreate(
-                name=name, description=description, **service_kwargs))
+        if self.cloud_config.get_api_version('identity').startswith('2'):
+            kwargs['service_type'] = type_ or service_type
+        else:
+            kwargs['type'] = type_ or service_type
+            kwargs['enabled'] = enabled
+
+        with _utils.shade_exceptions(
+            "Failed to create service {name}".format(name=name)
+        ):
+            service = self.manager.submitTask(
+                _tasks.ServiceCreate(name=name, **kwargs)
+            )
+
+        return _utils.normalize_keystone_services([service])[0]
+
+    @_utils.valid_kwargs('name', 'enabled', 'type', 'service_type',
+                         'description')
+    def update_service(self, name_or_id, **kwargs):
+        # NOTE(SamYaple): Service updates are only available on v3 api
+        if self.cloud_config.get_api_version('identity').startswith('2'):
+            raise OpenStackCloudUnavailableFeature(
+                'Unavailable Feature: Service update requires Identity v3'
+            )
+
+        # NOTE(SamYaple): Keystone v3 only accepts 'type' but shade accepts
+        #                 both 'type' and 'service_type' with a preference
+        #                 towards 'type'
+        type_ = kwargs.pop('type', None)
+        service_type = kwargs.pop('service_type', None)
+        if type_ or service_type:
+            kwargs['type'] = type_ or service_type
+
+        with _utils.shade_exceptions(
+            "Error in updating service {service}".format(service=name_or_id)
+        ):
+            service = self.manager.submitTask(
+                _tasks.ServiceUpdate(service=name_or_id, **kwargs)
+            )
 
         return _utils.normalize_keystone_services([service])[0]
 
@@ -875,9 +905,13 @@ class OperatorCloud(openstackcloud.OpenStackCloud):
         :raises: OpenStackCloudException if the service cannot be found or if
             something goes wrong during the openstack API call.
         """
-        if url and kwargs:
+        public_url = kwargs.pop('public_url', None)
+        internal_url = kwargs.pop('internal_url', None)
+        admin_url = kwargs.pop('admin_url', None)
+
+        if (url or interface) and (public_url or internal_url or admin_url):
             raise OpenStackCloudException(
-                "create_endpoint takes either url and interace OR"
+                "create_endpoint takes either url and interface OR"
                 " public_url, internal_url, admin_url")
 
         service = self.get_service(name_or_id=service_name_or_id)
@@ -898,40 +932,49 @@ class OperatorCloud(openstackcloud.OpenStackCloud):
                         " internal_url, admin_url parameters instead of"
                         " url and interface".format(
                             service=service_name_or_id))
-                urlkwargs['%url' % interface] = url
-                urlkwargs['service_id'] = service['id']
+                urlkwargs['{}url'.format(interface)] = url
             else:
                 urlkwargs['url'] = url
                 urlkwargs['interface'] = interface
-                urlkwargs['enabled'] = enabled
-                urlkwargs['service'] = service['id']
             endpoint_args.append(urlkwargs)
         else:
-            if self.cloud_config.get_api_version(
-                    'identity').startswith('2'):
+            expected_endpoints = {'public': public_url,
+                                  'internal': internal_url,
+                                  'admin': admin_url}
+            if self.cloud_config.get_api_version('identity').startswith('2'):
                 urlkwargs = {}
-                for arg_key, arg_val in kwargs.items():
-                    urlkwargs[arg_key.replace('_', '')] = arg_val
-                urlkwargs['service_id'] = service['id']
+                for interface, url in expected_endpoints.items():
+                    if url:
+                        urlkwargs['{}url'.format(interface)] = url
                 endpoint_args.append(urlkwargs)
             else:
-                for arg_key, arg_val in kwargs.items():
-                    urlkwargs = {}
-                    urlkwargs['url'] = arg_val
-                    urlkwargs['interface'] = arg_key.split('_')[0]
-                    urlkwargs['enabled'] = enabled
-                    urlkwargs['service'] = service['id']
-                    endpoint_args.append(urlkwargs)
+                for interface, url in expected_endpoints.items():
+                    if url:
+                        urlkwargs = {}
+                        urlkwargs['url'] = url
+                        urlkwargs['interface'] = interface
+                        endpoint_args.append(urlkwargs)
+
+        if self.cloud_config.get_api_version('identity').startswith('2'):
+            kwargs['service_id'] = service['id']
+            # Keystone v2 requires 'region' arg even if it is None
+            kwargs['region'] = region
+        else:
+            kwargs['service'] = service['id']
+            kwargs['enabled'] = enabled
+            if region is not None:
+                kwargs['region'] = region
 
         with _utils.shade_exceptions(
-            "Failed to create endpoint for service "
-            "{service}".format(service=service['name'])
+            "Failed to create endpoint for service"
+            " {service}".format(service=service['name'])
         ):
             for args in endpoint_args:
-                endpoint = self.manager.submitTask(_tasks.EndpointCreate(
-                    region=region,
-                    **args
-                ))
+                # NOTE(SamYaple): Add shared kwargs to endpoint args
+                args.update(kwargs)
+                endpoint = self.manager.submitTask(
+                    _tasks.EndpointCreate(**args)
+                )
                 endpoints.append(endpoint)
             return endpoints
 
@@ -1385,7 +1428,7 @@ class OperatorCloud(openstackcloud.OpenStackCloud):
                                     is_public=is_public)
             )
 
-        return flavor
+        return _utils.normalize_flavors([flavor])[0]
 
     def delete_flavor(self, name_or_id):
         """Delete a flavor
@@ -1408,47 +1451,6 @@ class OperatorCloud(openstackcloud.OpenStackCloud):
 
         return True
 
-    def _mod_flavor_specs(self, action, flavor_id, specs):
-        """Common method for modifying flavor extra specs.
-
-        Nova (very sadly) doesn't expose this with a public API, so we
-        must get the actual flavor object and make a method call on it.
-
-        Two separate try-except blocks are used because Nova can raise
-        a NotFound exception if FlavorGet() is given an invalid flavor ID,
-        or if the unset_keys() method of the flavor object is given an
-        invalid spec key. We need to be able to differentiate between these
-        actions, thus the separate blocks.
-        """
-        try:
-            flavor = self.manager.submitTask(
-                _tasks.FlavorGet(flavor=flavor_id)
-            )
-        except nova_exceptions.NotFound:
-            self.log.debug(
-                "Flavor ID {0} not found. "
-                "Cannot {1} extra specs.".format(flavor_id, action)
-            )
-            raise OpenStackCloudResourceNotFound(
-                "Flavor ID {0} not found".format(flavor_id)
-            )
-        except OpenStackCloudException:
-            raise
-        except Exception as e:
-            raise OpenStackCloudException(
-                "Error getting flavor ID {0}: {1}".format(flavor_id, e)
-            )
-
-        try:
-            if action == 'set':
-                flavor.set_keys(specs)
-            elif action == 'unset':
-                flavor.unset_keys(specs)
-        except Exception as e:
-            raise OpenStackCloudException(
-                "Unable to {0} flavor specs: {1}".format(action, e)
-            )
-
     def set_flavor_specs(self, flavor_id, extra_specs):
         """Add extra specs to a flavor
 
@@ -1458,7 +1460,14 @@ class OperatorCloud(openstackcloud.OpenStackCloud):
         :raises: OpenStackCloudException on operation error.
         :raises: OpenStackCloudResourceNotFound if flavor ID is not found.
         """
-        self._mod_flavor_specs('set', flavor_id, extra_specs)
+        try:
+            self.manager.submitTask(
+                _tasks.FlavorSetExtraSpecs(
+                    id=flavor_id, json=dict(extra_specs=extra_specs)))
+        except Exception as e:
+            raise OpenStackCloudException(
+                "Unable to set flavor specs: {0}".format(str(e))
+            )
 
     def unset_flavor_specs(self, flavor_id, keys):
         """Delete extra specs from a flavor
@@ -1469,7 +1478,14 @@ class OperatorCloud(openstackcloud.OpenStackCloud):
         :raises: OpenStackCloudException on operation error.
         :raises: OpenStackCloudResourceNotFound if flavor ID is not found.
         """
-        self._mod_flavor_specs('unset', flavor_id, keys)
+        for key in keys:
+            try:
+                self.manager.submitTask(
+                    _tasks.FlavorUnsetExtraSpecs(id=flavor_id, key=key))
+            except Exception as e:
+                raise OpenStackCloudException(
+                    "Unable to delete flavor spec {0}: {0}".format(
+                        key, str(e)))
 
     def _mod_flavor_access(self, action, flavor_id, project_id):
         """Common method for adding and removing flavor access
@@ -1582,7 +1598,7 @@ class OperatorCloud(openstackcloud.OpenStackCloud):
         :param string user: The name or id of the user.
         :param string group: The name or id of the group. (v3)
         :param string project: The name or id of the project.
-        :param string domain: The name or id of the domain. (v3)
+        :param string domain: The id of the domain. (v3)
         :param bool wait: Wait for role to be granted
         :param int timeout: Timeout to wait for role to be granted
 
