@@ -13,7 +13,6 @@
 # limitations under the License.
 
 import mock
-import testtools
 import warlock
 
 from neutronclient.common import exceptions as neutron_exceptions
@@ -22,6 +21,7 @@ import shade
 from shade import _utils
 from shade import meta
 from shade.tests import fakes
+from shade.tests.unit import base
 
 PRIVATE_V4 = '198.51.100.3'
 PUBLIC_V4 = '192.0.2.99'
@@ -65,22 +65,25 @@ class FakeCloud(object):
         return []
 
 
-class FakeServer(object):
-    id = 'test-id-0'
-    metadata = {'group': 'test-group'}
-    addresses = {'private': [{'OS-EXT-IPS:type': 'fixed',
-                              'addr': PRIVATE_V4,
-                              'version': 4}],
-                 'public': [{'OS-EXT-IPS:type': 'floating',
-                             'addr': PUBLIC_V4,
-                             'version': 4}]}
-    flavor = {'id': '101'}
-    image = {'id': '471c2475-da2f-47ac-aba5-cb4aa3d546f5'}
-    accessIPv4 = ''
-    accessIPv6 = ''
+standard_fake_server = fakes.FakeServer(
+    id='test-id-0',
+    name='test-id-0',
+    status='ACTIVE',
+    metadata={'group': 'test-group'},
+    addresses={'private': [{'OS-EXT-IPS:type': 'fixed',
+                            'addr': PRIVATE_V4,
+                            'version': 4}],
+               'public': [{'OS-EXT-IPS:type': 'floating',
+                           'addr': PUBLIC_V4,
+                           'version': 4}]},
+    flavor={'id': '101'},
+    image={'id': '471c2475-da2f-47ac-aba5-cb4aa3d546f5'},
+    accessIPv4='',
+    accessIPv6='',
+)
 
 
-class TestMeta(testtools.TestCase):
+class TestMeta(base.TestCase):
     def test_find_nova_addresses_key_name(self):
         # Note 198.51.100.0/24 is TEST-NET-2 from rfc5737
         addrs = {'public': [{'addr': '198.51.100.1', 'version': 4}],
@@ -121,10 +124,11 @@ class TestMeta(testtools.TestCase):
             addrs, key_name='public', ext_tag='fixed', version=6))
 
     def test_get_server_ip(self):
-        srv = meta.obj_to_dict(FakeServer())
-        cloud = shade.openstack_cloud(validate=False)
-        self.assertEqual(PRIVATE_V4, meta.get_server_private_ip(srv, cloud))
-        self.assertEqual(PUBLIC_V4, meta.get_server_external_ipv4(cloud, srv))
+        srv = meta.obj_to_dict(standard_fake_server)
+        self.assertEqual(
+            PRIVATE_V4, meta.get_server_ip(srv, ext_tag='fixed'))
+        self.assertEqual(
+            PUBLIC_V4, meta.get_server_ip(srv, ext_tag='floating'))
 
     @mock.patch.object(shade.OpenStackCloud, 'has_service')
     @mock.patch.object(shade.OpenStackCloud, 'search_networks')
@@ -145,15 +149,16 @@ class TestMeta(testtools.TestCase):
                                    'addr': PUBLIC_V4,
                                    'version': 4}]}
         ))
-        cloud = shade.openstack_cloud(validate=False)
 
-        self.assertEqual(PRIVATE_V4, meta.get_server_private_ip(srv, cloud))
+        self.assertEqual(
+            PRIVATE_V4, meta.get_server_private_ip(srv, self.cloud))
         mock_has_service.assert_called_with('network')
         mock_search_networks.assert_called_with(
             filters={'router:external': False}
         )
 
     @mock.patch.object(shade.OpenStackCloud, 'list_server_security_groups')
+    @mock.patch.object(shade.OpenStackCloud, 'get_volumes')
     @mock.patch.object(shade.OpenStackCloud, 'get_image_name')
     @mock.patch.object(shade.OpenStackCloud, 'get_flavor_name')
     @mock.patch.object(shade.OpenStackCloud, 'has_service')
@@ -161,10 +166,12 @@ class TestMeta(testtools.TestCase):
     def test_get_server_private_ip_devstack(
             self, mock_search_networks, mock_has_service,
             mock_get_flavor_name, mock_get_image_name,
+            mock_get_volumes,
             mock_list_server_security_groups):
         mock_get_image_name.return_value = 'cirros-0.3.4-x86_64-uec'
         mock_get_flavor_name.return_value = 'm1.tiny'
         mock_has_service.return_value = True
+        mock_get_volumes.return_value = []
         mock_search_networks.return_value = [
             {
                 'id': 'test_pnztt_net',
@@ -176,8 +183,7 @@ class TestMeta(testtools.TestCase):
             },
         ]
 
-        cloud = shade.openstack_cloud(validate=False)
-        srv = cloud.get_openstack_vars(meta.obj_to_dict(fakes.FakeServer(
+        srv = self.cloud.get_openstack_vars(meta.obj_to_dict(fakes.FakeServer(
             id='test-id', name='test-name', status='ACTIVE',
             flavor={u'id': u'1'},
             image={
@@ -207,7 +213,8 @@ class TestMeta(testtools.TestCase):
         mock_has_service.return_value = True
         mock_search_networks.return_value = [{
             'id': 'test-net-id',
-            'name': 'test-net'
+            'name': 'test-net',
+            'router:external': True,
         }]
 
         srv = meta.obj_to_dict(fakes.FakeServer(
@@ -216,17 +223,61 @@ class TestMeta(testtools.TestCase):
                 'addr': PUBLIC_V4,
                 'version': 4}]},
         ))
-        ip = meta.get_server_external_ipv4(
-            cloud=shade.openstack_cloud(validate=False), server=srv)
+        ip = meta.get_server_external_ipv4(cloud=self.cloud, server=srv)
 
         self.assertEqual(PUBLIC_V4, ip)
 
+    @mock.patch.object(shade.OpenStackCloud, 'has_service')
+    @mock.patch.object(shade.OpenStackCloud, 'search_networks')
+    def test_get_server_external_provider_ipv4_neutron(
+            self, mock_search_networks,
+            mock_has_service):
+        # Testing Clouds with Neutron
+        mock_has_service.return_value = True
+        mock_search_networks.return_value = [{
+            'id': 'test-net-id',
+            'name': 'test-net',
+            'provider:network_type': 'vlan',
+        }]
+
+        srv = meta.obj_to_dict(fakes.FakeServer(
+            id='test-id', name='test-name', status='ACTIVE',
+            addresses={'test-net': [{
+                'addr': PUBLIC_V4,
+                'version': 4}]},
+        ))
+        ip = meta.get_server_external_ipv4(cloud=self.cloud, server=srv)
+
+        self.assertEqual(PUBLIC_V4, ip)
+
+    @mock.patch.object(shade.OpenStackCloud, 'has_service')
+    @mock.patch.object(shade.OpenStackCloud, 'search_networks')
+    def test_get_server_external_none_ipv4_neutron(
+            self, mock_search_networks,
+            mock_has_service):
+        # Testing Clouds with Neutron
+        mock_has_service.return_value = True
+        mock_search_networks.return_value = [{
+            'id': 'test-net-id',
+            'name': 'test-net',
+            'router:external': False,
+        }]
+
+        srv = meta.obj_to_dict(fakes.FakeServer(
+            id='test-id', name='test-name', status='ACTIVE',
+            addresses={'test-net': [{
+                'addr': PUBLIC_V4,
+                'version': 4}]},
+        ))
+        ip = meta.get_server_external_ipv4(cloud=self.cloud, server=srv)
+
+        self.assertEqual(None, ip)
+
     def test_get_server_external_ipv4_neutron_accessIPv4(self):
         srv = meta.obj_to_dict(fakes.FakeServer(
             id='test-id', name='test-name', status='ACTIVE',
             accessIPv4=PUBLIC_V4))
-        ip = meta.get_server_external_ipv4(
-            cloud=shade.openstack_cloud(validate=False), server=srv)
+        ip = meta.get_server_external_ipv4(cloud=self.cloud, server=srv)
 
         self.assertEqual(PUBLIC_V4, ip)
 
@@ -254,8 +305,7 @@ class TestMeta(testtools.TestCase):
 
         srv = meta.obj_to_dict(fakes.FakeServer(
             id='test-id', name='test-name', status='ACTIVE'))
-        ip = meta.get_server_external_ipv4(
-            cloud=shade.openstack_cloud(validate=False), server=srv)
+        ip = meta.get_server_external_ipv4(cloud=self.cloud, server=srv)
 
         self.assertEqual(PUBLIC_V4, ip)
         self.assertTrue(mock_get_server_ip.called)
@@ -269,8 +319,7 @@ class TestMeta(testtools.TestCase):
         srv = meta.obj_to_dict(fakes.FakeServer(
             id='test-id', name='test-name', status='ACTIVE',
             addresses={'public': [{'addr': PUBLIC_V4, 'version': 4}]}))
-        ip = meta.get_server_external_ipv4(
-            cloud=shade.openstack_cloud(validate=False), server=srv)
+        ip = meta.get_server_external_ipv4(cloud=self.cloud, server=srv)
 
         self.assertEqual(PUBLIC_V4, ip)
 
@@ -285,8 +334,7 @@ class TestMeta(testtools.TestCase):
         srv = meta.obj_to_dict(fakes.FakeServer(
             id='test-id', name='test-name', status='ACTIVE',
             addresses={'test-net': [{'addr': PRIVATE_V4}]}))
-        ip = meta.get_server_external_ipv4(
-            cloud=shade.openstack_cloud(validate=False), server=srv)
+        ip = meta.get_server_external_ipv4(cloud=self.cloud, server=srv)
 
         self.assertIsNone(ip)
         self.assertTrue(mock_get_server_ip.called)
@@ -319,7 +367,11 @@ class TestMeta(testtools.TestCase):
              'test-region_test-az',
              'test-name_test-region_test-az'],
             meta.get_groups_from_server(
-                FakeCloud(), meta.obj_to_dict(FakeServer()), server_vars))
+                FakeCloud(),
+                meta.obj_to_dict(standard_fake_server),
+                server_vars
+            )
+        )
 
     def test_obj_list_to_dict(self):
         """Test conversion of a list of objects to a list of dictonaries"""
@@ -344,7 +396,7 @@ class TestMeta(testtools.TestCase):
         mock_list_server_security_groups.return_value = [
             {'name': 'testgroup', 'id': '1'}]
 
-        server = meta.obj_to_dict(FakeServer())
+        server = meta.obj_to_dict(standard_fake_server)
         hostvars = meta.get_hostvars_from_server(FakeCloud(), server)
 
         mock_list_server_security_groups.assert_called_once_with(server)
@@ -361,7 +413,7 @@ class TestMeta(testtools.TestCase):
 
         hostvars = meta.get_hostvars_from_server(
             FakeCloud(), _utils.normalize_server(
-                meta.obj_to_dict(FakeServer()),
+                meta.obj_to_dict(standard_fake_server),
                 cloud_name='CLOUD_NAME',
                 region_name='REGION_NAME'))
         self.assertNotIn('links', hostvars)
@@ -372,9 +424,11 @@ class TestMeta(testtools.TestCase):
         self.assertEquals('REGION_NAME', hostvars['region'])
         self.assertEquals('CLOUD_NAME', hostvars['cloud'])
         self.assertEquals("test-image-name", hostvars['image']['name'])
-        self.assertEquals(FakeServer.image['id'], hostvars['image']['id'])
+        self.assertEquals(standard_fake_server.image['id'],
+                          hostvars['image']['id'])
         self.assertNotIn('links', hostvars['image'])
-        self.assertEquals(FakeServer.flavor['id'], hostvars['flavor']['id'])
+        self.assertEquals(standard_fake_server.flavor['id'],
+                          hostvars['flavor']['id'])
         self.assertEquals("test-flavor-name", hostvars['flavor']['name'])
         self.assertNotIn('links', hostvars['flavor'])
         # test having volumes
@@ -392,7 +446,7 @@ class TestMeta(testtools.TestCase):
         fake_cloud = FakeCloud()
         fake_cloud.force_ipv4 = True
         hostvars = meta.get_hostvars_from_server(
-            fake_cloud, meta.obj_to_dict(FakeServer()))
+            fake_cloud, meta.obj_to_dict(standard_fake_server))
         self.assertEqual(PUBLIC_V4, hostvars['interface_ip'])
 
     @mock.patch.object(shade.meta, 'get_server_external_ipv4')
@@ -402,21 +456,21 @@ class TestMeta(testtools.TestCase):
         cloud = FakeCloud()
         cloud.private = True
         hostvars = meta.get_hostvars_from_server(
-            cloud, meta.obj_to_dict(FakeServer()))
+            cloud, meta.obj_to_dict(standard_fake_server))
         self.assertEqual(PRIVATE_V4, hostvars['interface_ip'])
 
     @mock.patch.object(shade.meta, 'get_server_external_ipv4')
     def test_image_string(self, mock_get_server_external_ipv4):
         mock_get_server_external_ipv4.return_value = PUBLIC_V4
 
-        server = FakeServer()
+        server = standard_fake_server
         server.image = 'fake-image-id'
         hostvars = meta.get_hostvars_from_server(
             FakeCloud(), meta.obj_to_dict(server))
         self.assertEquals('fake-image-id', hostvars['image']['id'])
 
     def test_az(self):
-        server = FakeServer()
+        server = standard_fake_server
         server.__dict__['OS-EXT-AZ:availability_zone'] = 'az1'
 
         hostvars = _utils.normalize_server(
@@ -435,7 +489,7 @@ class TestMeta(testtools.TestCase):
         fake_volume_dict = meta.obj_to_dict(fake_volume)
         mock_cloud.get_volumes.return_value = [fake_volume_dict]
         hostvars = meta.get_hostvars_from_server(
-            mock_cloud, meta.obj_to_dict(FakeServer()))
+            mock_cloud, meta.obj_to_dict(standard_fake_server))
         self.assertEquals('volume1', hostvars['volumes'][0]['id'])
         self.assertEquals('/dev/sda0', hostvars['volumes'][0]['device'])
 
@@ -443,7 +497,7 @@ class TestMeta(testtools.TestCase):
         fake_cloud = FakeCloud()
         fake_cloud.service_val = False
         hostvars = meta.get_hostvars_from_server(
-            fake_cloud, meta.obj_to_dict(FakeServer()))
+            fake_cloud, meta.obj_to_dict(standard_fake_server))
         self.assertEquals([], hostvars['volumes'])
 
     def test_unknown_volume_exception(self):
@@ -459,11 +513,11 @@ class TestMeta(testtools.TestCase):
             FakeException,
             meta.get_hostvars_from_server,
             mock_cloud,
-            meta.obj_to_dict(FakeServer()))
+            meta.obj_to_dict(standard_fake_server))
 
     def test_obj_to_dict(self):
         cloud = FakeCloud()
-        cloud.server = FakeServer()
+        cloud.server = standard_fake_server
         cloud_dict = meta.obj_to_dict(cloud)
         self.assertEqual(FakeCloud.name, cloud_dict['name'])
         self.assertNotIn('_unused', cloud_dict)
